using System;
using System.Collections;
using UnityEngine;
using System.Collections.Generic;
using System.IO;
using System.Linq;
//using System.Runtime.InteropServices;

public class ModAPI2
{
    public static string metaString;

    public void setMetaString(string st)
    {
        metaString = st;
    }

    public static Sprite LoadSprite(string path, float scale = 1f, bool pixelated = true)
    {
        string str = Path.Combine(ModAPI.Metadata.MetaLocation, path);
        Sprite sprite2 = LoadSprite1(str, pixelated ? FilterMode.Point : FilterMode.Bilinear, scale * 35f, false);
        //ModResourceCache.Cache(str, (UnityEngine.Object)sprite2);
        return sprite2;
    }
    public static Sprite LoadSprite1(
    string fullPath,
    FilterMode mode = FilterMode.Bilinear,
    float pixelsPerUnit = 35f,
    bool markNonReadable = true)
    {
        Texture2D texture = Utils.LoadTexture(fullPath, mode, markNonReadable);
        return (UnityEngine.Object)texture == (UnityEngine.Object)null ? (Sprite)null : Sprite.Create(texture, new Rect(0.0f, 0.0f, (float)texture.width, (float)texture.height), 0.5f * Vector2.one, pixelsPerUnit);
    }

    public static AudioClip LoadSound(string path)
    {
        string str = Path.Combine(metaString, path);
        AudioClip audioClip1;
        if (ModResourceCache.TryGet<AudioClip>(str, out audioClip1))
            return audioClip1;
        AudioClip audioClip2 = Utils.FileToAudioClip(str);
        ModResourceCache.Cache(str, (UnityEngine.Object)audioClip2);
        return audioClip2;
    }
}

public class TrainCart : MonoBehaviour
{
    private void Start()
    {
        WheelJoint2D[] engines = GetComponentsInChildren<WheelJoint2D>();
        foreach (var engine in engines)
        {
            engine.gameObject.AddComponent<WheelBehaviour>();
            engine.gameObject.GetComponent<WheelBehaviour>().cart = gameObject;
        }
    }
    private void OnDestroy()
    {
        CouplerBehaviour[] couplers = GetComponentsInChildren<CouplerBehaviour>();
        foreach (CouplerBehaviour coupler in couplers)
        {
            coupler.connectedCoupler.disconnect();
            coupler.disconnect();
        }
    }
}

public class WheelBehaviour : MonoBehaviour
{
    public GameObject cart;
    Rigidbody2D rb, cartrb;
    public bool isSliding = false, isfricting = false;
    AudioSource slidingSource;
    float locomotiveVelocity, wheelTorque, timer;
    float down;

    private void Start()
    {
        down = gameObject.GetComponent<CircleCollider2D>().radius;
        cartrb = cart.GetComponent<Rigidbody2D>();
        slidingSource = gameObject.AddComponent<AudioSource>();
        rb = GetComponent<Rigidbody2D>();
        slidingSource.loop = true;
        slidingSource.clip = ModAPI2.LoadSound("sounds/brake.wav");
        slidingSource.volume = 0f;
        slidingSource.Play();
    }

    private void OnCollisionStay2D(Collision2D col)
    {
        isSliding = true;
        locomotiveVelocity = cartrb.velocity.magnitude;
        wheelTorque = rb.angularVelocity / 100f;
        if (Mathf.Abs(locomotiveVelocity) > Mathf.Abs(wheelTorque) + 4f || Mathf.Abs(locomotiveVelocity) < Mathf.Abs(wheelTorque) - 4f)
            isfricting = true;
        else
            isfricting = false;
    }
    private void OnCollisionExit2D(Collision2D col)
    {
        isSliding = false;
    }

    private void Update()
    {
        slidingSource.pitch = Time.timeScale;
        if (Input.GetKeyDown(KeyCode.L))
        {
            ModAPI.Notify(locomotiveVelocity);
            ModAPI.Notify(wheelTorque);
        }
        if (timer < 1)
            timer += Time.deltaTime;
        if (isSliding && isfricting)
        {
            slidingSource.volume = Mathf.Lerp(slidingSource.volume, 0.03f, 0.04f);
            if (timer > 0.1f)
            {
                timer = 0;
                GameObject yee = ModAPI.CreateParticleEffect("BigZap", transform.position + -Vector3.up * down);
                //yee.transform.localScale = new Vector2(1f, 1f);
                Destroy(yee.gameObject.GetComponent<AudioSource>());
            }
        }
        else
        {
            slidingSource.volume = Mathf.Lerp(slidingSource.volume, 0f, 0.04f);
        }
    }
}

public class Locomotive : MonoBehaviour
{
    public int health = 300;
    public float maxSpeed = 129f;
    public float currentMaxSpeed = 0f;
    public float power = 1200f;
    public float suspensionValue = 20f;
    public float accelation = 60f;
    public bool isSteam = false;
    public int coals = 0;
    public bool mute = false;
    public bool parking = true;

    public AudioClip horn;
    public AudioClip release;
    public AudioClip brush;
    public AudioClip loop;
    public AudioClip run;
    public AudioSource S_horn;
    public AudioSource S_release;
    public AudioSource S_brush;
    public AudioSource S_loop;
    public AudioSource S_run;

    JointSuspension2D jointMotor;
    JointMotor2D setting;
    float force = 100f;
    float range = 40f;
    float currentVelocity = 0;
    bool brake = true;
    bool reversed = false;
    WheelJoint2D[] engines;
    private void Awake()
    {
        if (transform.localScale.x < 0)
            reversed = true;
        GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("horn", "horn", "horn", () =>
        {
            S_horn.Play();
        }));
        GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("reverse", "reverse gear", "reverse gear", () =>
        {
            ModAPI.Notify("direction reversed");
            S_horn.PlayOneShot(ModAPI2.LoadSound("sounds/connect.wav"));
            reversed = !reversed;
            currentVelocity *= -1f;
        }));
        GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("mute", "mute/unmute locomotive", "horn", () =>
        {
            mute = !mute;
            AudioSource[] everything = GetComponents<AudioSource>();
            foreach (AudioSource a in everything)
                a.enabled = !mute;
        }));
    }
    private void Start()
    {
        S_horn = gameObject.AddComponent<AudioSource>();
        S_release = gameObject.AddComponent<AudioSource>();
        S_brush = gameObject.AddComponent<AudioSource>();
        S_loop = gameObject.AddComponent<AudioSource>();
        S_run = gameObject.AddComponent<AudioSource>();
        S_horn.clip = horn;
        S_release.clip = release;
        S_brush.clip = brush;
        S_loop.clip = loop;
        S_loop.loop = true;
        S_run.clip = run;
        S_run.loop = true;
        S_horn.volume *= 0.4f;
        S_release.volume *= 0.4f;
        S_brush.volume *= 0.4f;
        S_loop.volume *= 0.4f;
        S_run.volume *= 0.4f;
        maxSpeed *= 8f;
        power *= 0.05f;
        engines = GetComponentsInChildren<WheelJoint2D>();
        jointMotor.frequency = suspensionValue;
        jointMotor.dampingRatio = 0;
        setting.motorSpeed = currentVelocity;
        setting.maxMotorTorque = power / engines.Length;
        currentMaxSpeed = maxSpeed;
        foreach (var engine in engines)
        {
            engine.motor = setting;
            engine.suspension = jointMotor;
            engine.useMotor = false;
            engine.gameObject.GetComponent<Rigidbody2D>().sharedMaterial.bounciness = 0;
            engine.gameObject.GetComponent<Rigidbody2D>().sharedMaterial.friction = 1;
        }
    }
    private void Update()
    {
        S_loop.pitch = Time.timeScale;
        S_run.pitch = Time.timeScale;
        S_brush.pitch = Time.timeScale;
        S_horn.pitch = Time.timeScale;
        S_release.pitch = Time.timeScale;
        if (!brake)
        {
            //not reversed
            if(!reversed)
            {
                if(currentVelocity < maxSpeed)
                    currentVelocity += 200 * Time.deltaTime;
                setting.motorSpeed = currentVelocity;
                foreach (var engine in engines)
                {
                    engine.motor = setting;
                }
            }
            //reversed
            else if (reversed)
            {
                if (-currentVelocity < maxSpeed)
                    currentVelocity -= 100 * Time.deltaTime;
                setting.motorSpeed = currentVelocity;
                foreach (var engine in engines)
                {
                    engine.motor = setting;
                }
            }
        }
    }
    private void Shot(global::Shot shot)
    {
        health -= 1;
        if(health <= 0)
        {
            //explode();
        }
    }
    public void switchEngine()
    {
        if(!brake)
        {
            setting.motorSpeed = 0;
            foreach (var engine in engines)
                engine.motor = setting;
        }
        if (brake && !mute)
        {
            S_release.Play();
            S_run.Play();
        }
        else
        {
            S_run.Stop();
        }
        brake = !brake;
        foreach (var engine in engines)
        {
            engine.useMotor = !engine.useMotor;
            engine.gameObject.GetComponent<Rigidbody2D>().freezeRotation = brake;
            currentVelocity = 0;
        }
    }
    public void explode()
    {
        if (!brake)
            switchEngine();
        GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Flammable metal");
        ModAPI.CreateParticleEffect("EnormousExplosion", transform.position);
        ExplosionCreator.CreatePulseExplosion(transform.position, force, range, true, true);
        ExplosionCreator.CreateFragmentationExplosion(1, transform.position, range, force, true, true, 1f);
        Destroy(this);
    }
    private void OnCollisionEnter2D(Collision2D col)
    {
        if(isSteam && col.gameObject.GetComponent<CoalBehaviour>())
        {
            Destroy(col.gameObject);
            addCoal();
        }
    }
    private void addCoal()
    {
        coals += 1;
        currentMaxSpeed += maxSpeed * 0.2f;
        //if (coals > 5)
        //    explode();
    }
}

public class CoalBehaviour : MonoBehaviour
{

}

public class CouplerBehaviour : MonoBehaviour
{
    AudioSource connect;
    AudioClip connection, disconnection;
    public CouplerBehaviour connectedCoupler;
    Transform parent;
    float timer = 0;
    private void Awake()
    {
        connection = ModAPI2.LoadSound("sounds/connect.wav");
        disconnection = ModAPI2.LoadSound("sounds/disconnect.wav");
        gameObject.AddComponent<FixedJoint2D>();
        connect = gameObject.AddComponent<AudioSource>();
        gameObject.GetComponent<FixedJoint2D>().connectedBody = gameObject.transform.parent.GetComponent<Rigidbody2D>();
        gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
        gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
        gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
        this.GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("disconnect", "disconnect", "disconnect", () =>
        {
            if (connectedCoupler)
            {
                disconnect();
                connectedCoupler.disconnect();
            }
        }));
    }

    public void disconnect()
    {
        if (GetComponent<HingeJoint2D>())
        {
            Rigidbody2D me = GetComponent<Rigidbody2D>();
            Rigidbody2D other = GetComponent<HingeJoint2D>().connectedBody;
            connect.PlayOneShot(connection);
            Destroy(GetComponent<HingeJoint2D>());
            me.velocity = me.gameObject.transform.position - other.gameObject.transform.position;
            other.velocity = other.gameObject.transform.position - me.gameObject.transform.position;
            timer = 0;
            other.gameObject.GetComponent<CouplerBehaviour>().timer = 0;
            other.gameObject.GetComponent<CouplerBehaviour>().connectedCoupler = null;
            connectedCoupler = null;
        }
    }

    private void Update()
    {
        if(timer < 2)
            timer += Time.deltaTime;
    }
    private void OnCollisionEnter2D(Collision2D col)
    {
        if (timer > 1 && col.gameObject.GetComponent<CouplerBehaviour>() != null && gameObject.GetComponent<HingeJoint2D>() == null && connectedCoupler == null && col.gameObject.GetComponent<CouplerBehaviour>().connectedCoupler == null)
        {
            ModAPI.Notify("connected");
            connect.PlayOneShot(disconnection);
            gameObject.AddComponent<HingeJoint2D>();
            gameObject.GetComponent<HingeJoint2D>().connectedBody = col.gameObject.GetComponent<Rigidbody2D>();
            if(gameObject.name == "frontCoupler")
                gameObject.GetComponent<HingeJoint2D>().anchor = new Vector2(0.7f, 0f);
            else
                gameObject.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.7f, 0f);
            gameObject.GetComponent<HingeJoint2D>().enableCollision = false;
            connectedCoupler = col.gameObject.GetComponent<CouplerBehaviour>();
            col.gameObject.GetComponent<CouplerBehaviour>().connectedCoupler = GetComponent<CouplerBehaviour>();
        }
    }
    private void OnDestroy()
    {
        disconnect();
    }
}

public class ExplodableCart : MonoBehaviour
{
    int health = 30;
    float force = 400f;
    float range = 40f;

    private void Shot(global::Shot shot)
    {
        health -= 1;
        if (health <= 0)
        {
            explode();
        }
    }
    public void explode()
    {
        GetComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Flammable metal");
        ModAPI.CreateParticleEffect("EnormousExplosion", transform.position);
        ExplosionCreator.CreatePulseExplosion(transform.position, force, range, true, true);
        ExplosionCreator.CreateFragmentationExplosion(1, transform.position, range, force, true, true, 1f);
        GetComponent<PhysicalBehaviour>().Ignite(true);
        Destroy(this);
    }
}

public class BasicDieselLocomotiveSetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    public float bogieDownY = -2f;
    public float frontBogieX = 4.715f;
    public float backBogieX = -4.97f;
    public float couplerInterval = 11.2f;
    public float interval = 2.255f;
    WheelJoint2D[] wheels;

    private void Awake()
    {
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel.png"));
        var frontWheel_3 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel.png"));
        var backWheel_3 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCouplerFreight.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));

        frontWheel_1.transform.position = transform.position + new Vector3(frontBogieX + 2 * interval, bogieDownY, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(frontBogieX + interval, bogieDownY, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        frontWheel_3.transform.position = transform.position + new Vector3(frontBogieX, bogieDownY, 0);
        frontWheel_3.transform.SetParent(transform);
        frontWheel_3.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(backBogieX, bogieDownY, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(backBogieX - interval, bogieDownY, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();
        backWheel_3.transform.position = transform.position + new Vector3(backBogieX - 2 * interval, bogieDownY, 0);
        backWheel_3.transform.SetParent(transform);
        backWheel_3.AddComponent<WheelJoint2D>();

        frontCoupler.transform.position = transform.position + new Vector3(couplerInterval, bogieDownY + 0.5f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-couplerInterval, bogieDownY + 0.5f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
        }
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(frontBogieX + 2 * interval, bogieDownY, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(frontBogieX + interval, bogieDownY, 0);
        frontWheel_3.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(frontBogieX, bogieDownY, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(backBogieX, bogieDownY, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(backBogieX - interval, bogieDownY, 0);
        backWheel_3.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(backBogieX - 2 * interval, bogieDownY, 0);
    }
}

public class C62SetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    public WheelJoint2D drivingWheel;
    public GameObject rod;
    float timer;
    float mult = 1.3f;
    bool readyForBrush = false;
    WheelJoint2D[] wheels;

    private void Awake()
    {
        GetComponent<SpriteRenderer>().sortingOrder = 3;
        var mainWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamWheel.png"));
        var mainWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamWheel.png"));
        var mainWheel_3 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamWheel.png"));
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCouplerFreight.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));
        rod = new GameObject();

        mainWheel_1.transform.position = transform.position + new Vector3(-1.88f, -1.3f, 0);
        mainWheel_1.transform.SetParent(transform);
        mainWheel_1.AddComponent<WheelJoint2D>();
        mainWheel_2.transform.position = transform.position + new Vector3(0.51f, -1.3f, 0);
        mainWheel_2.transform.SetParent(transform);
        mainWheel_2.AddComponent<WheelJoint2D>();
        mainWheel_3.transform.position = transform.position + new Vector3(2.85f, -1.3f, 0);
        mainWheel_3.transform.SetParent(transform);
        mainWheel_3.AddComponent<WheelJoint2D>();

        frontWheel_1.transform.position = transform.position + new Vector3(4.6f, -1.816f, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(6.147f, -1.816f, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(-5.51f, -1.816f, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(-4.11f, -1.816f, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();

        rod.transform.position = transform.position + new Vector3(1.12f, -0.84f, 0);
        rod.transform.SetParent(transform);
        rod.gameObject.AddComponent<SpriteRenderer>();
        rod.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 2;
        if (transform.localScale.x < 0) //facing left
        {
            rod.transform.localScale = new Vector2(-rod.transform.localScale.x, rod.transform.localScale.y);
            rod.transform.localPosition = new Vector3(-rod.transform.localPosition.x, rod.transform.localPosition.y, rod.transform.localPosition.z);
        }

        frontCoupler.transform.position = transform.position + new Vector3(7.836f, -1.04f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-6.635f, -0.644f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();

        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
        }
        mainWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-1.88f, -1.3f, 0);
        mainWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(0.51f, -1.3f, 0);
        mainWheel_3.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(2.85f, -1.3f, 0);

        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(4.6f, -1.816f, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(6.147f, -1.816f, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-5.71f, -1.816f, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-4.31f, -1.816f, 0);
        drivingWheel = mainWheel_3.GetComponent<WheelJoint2D>();
    }
    private void brushSound()
    {
        if(readyForBrush)
        {
            readyForBrush = false;
            GetComponent<Locomotive>().S_brush.Play();
        }
    }
    private void Update()
    {
        if(drivingWheel.useMotor)
        {
            timer += Time.deltaTime;
            if(timer > 0.2f)
            {
                timer = 0;
                if (GetComponent<Locomotive>().coals == 0)
                    return;
                var steam = new GameObject();
                steam.transform.SetParent(transform);
                steam.transform.localPosition = new Vector3(5.818f, 2.344f, 0);
                steam.transform.SetParent(null);
                steam.gameObject.AddComponent<SpriteRenderer>();
                steam.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("steam/steam1.png");
                steam.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
                steam.gameObject.AddComponent<Steam>();
                var steam2 = new GameObject();
                steam2.transform.SetParent(transform);
                steam2.transform.localPosition = new Vector3(4.25f, -1.43f, 0);
                steam2.transform.SetParent(null);
                steam2.gameObject.AddComponent<SpriteRenderer>();
                steam2.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("steam/steam1.png");
                steam2.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 4;
                steam2.gameObject.AddComponent<Steam2>();
            }
        }
        if (drivingWheel.transform.localRotation.eulerAngles.z > 315f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("C62_rods/rod0.png", mult);
        else if (drivingWheel.transform.localRotation.eulerAngles.z > 270f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("C62_rods/rod45.png", mult);
        else if (drivingWheel.transform.localRotation.eulerAngles.z > 225f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("C62_rods/rod90.png", mult);
        else if (drivingWheel.transform.localRotation.eulerAngles.z > 180f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("C62_rods/rod135.png", mult);
        else if (drivingWheel.transform.localRotation.eulerAngles.z > 135f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("C62_rods/rod180.png", mult);
        else if (drivingWheel.transform.localRotation.eulerAngles.z > 90f)
        {
            readyForBrush = true;
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("C62_rods/rod225.png", mult);
        }
        else if (drivingWheel.transform.localRotation.eulerAngles.z > 45f)
        {
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("C62_rods/rod270.png", mult);
            brushSound();
        }
        else
        {
            readyForBrush = true;
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("C62_rods/rod315.png", mult);
        }
    }
}

public class MarquetteSetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    public WheelJoint2D drivingWheel;
    private GameObject rod;
    float timer;
    private float mult = 0.769f;
    bool readyForBrush = false;
    WheelJoint2D[] wheels;

    private void Awake()
    {
        GetComponent<SpriteRenderer>().sortingOrder = 3;
        var mainWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamWheel.png"));
        var mainWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamWheel.png"));
        var mainWheel_3 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamWheel.png"));
        var mainWheel_4 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamWheel.png"));
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCouplerFreight.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));
        rod = new GameObject();

        mainWheel_1.transform.position = transform.position + new Vector3(-3.384f, -1.706f, 0);
        mainWheel_1.transform.SetParent(transform);
        mainWheel_1.AddComponent<WheelJoint2D>();
        mainWheel_2.transform.position = transform.position + new Vector3(-0.69f, -1.706f, 0);
        mainWheel_2.transform.SetParent(transform);
        mainWheel_2.AddComponent<WheelJoint2D>();
        mainWheel_3.transform.position = transform.position + new Vector3(2.022f, -1.706f, 0);
        mainWheel_3.transform.SetParent(transform);
        mainWheel_3.AddComponent<WheelJoint2D>();
        mainWheel_4.transform.position = transform.position + new Vector3(4.734f, -1.706f, 0);
        mainWheel_4.transform.SetParent(transform);
        mainWheel_4.AddComponent<WheelJoint2D>();
        mainWheel_1.transform.localScale = new Vector2(1.2f, 1.2f);
        mainWheel_2.transform.localScale = new Vector2(1.2f, 1.2f);
        mainWheel_3.transform.localScale = new Vector2(1.2f, 1.2f);
        mainWheel_4.transform.localScale = new Vector2(1.2f, 1.2f);

        frontWheel_1.transform.position = transform.position + new Vector3(7.71f, -2.439f, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(-7.527f, -2.439f, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(-5.74f, -2.439f, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();

        rod.transform.position = transform.position + new Vector3(2.74f, -0.87f, 0);
        rod.transform.SetParent(transform);
        rod.transform.localScale = new Vector2(1.3f, 1.3f);
        rod.gameObject.AddComponent<SpriteRenderer>();
        rod.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 2;
        if (transform.localScale.x < 0) //facing left
        {
            rod.transform.localScale = new Vector2(-rod.transform.localScale.x, rod.transform.localScale.y);
            rod.transform.localPosition = new Vector3(-rod.transform.localPosition.x, rod.transform.localPosition.y, rod.transform.localPosition.z);
        }

        frontCoupler.transform.position = transform.position + new Vector3(9.83f, -1.888f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-10.52f, -1.888f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
        }
        mainWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-3.384f, -1.706f, 0);
        mainWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-0.69f, -1.706f, 0);
        mainWheel_3.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(2.022f, -1.706f, 0);
        mainWheel_4.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(4.734f, -1.706f, 0);
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(7.71f, -2.439f, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-7.527f, -2.439f, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-5.74f, -2.439f, 0);
        drivingWheel = mainWheel_4.GetComponent<WheelJoint2D>();
    }
    private void brushSound()
    {
        if (readyForBrush)
        {
            readyForBrush = false;
            GetComponent<Locomotive>().S_brush.Play();
        }
    }
    private void Update()
    {
        if (drivingWheel.useMotor)
        {
            timer += Time.deltaTime;
            if (timer > 0.2f)
            {
                timer = 0;
                if (GetComponent<Locomotive>().coals == 0)
                    return;
                var steam = new GameObject();
                steam.transform.SetParent(transform);
                steam.transform.localPosition = new Vector3(5.818f, 2.344f, 0);
                steam.transform.SetParent(null);
                steam.gameObject.AddComponent<SpriteRenderer>();
                steam.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("steam/steam1.png");
                steam.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
                steam.gameObject.AddComponent<Steam>();
                var steam2 = new GameObject();
                steam2.transform.SetParent(transform);
                steam2.transform.localPosition = new Vector3(4.25f, -1.43f, 0);
                steam2.transform.SetParent(null);
                steam2.gameObject.AddComponent<SpriteRenderer>();
                steam2.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("steam/steam1.png");
                steam2.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 4;
                steam2.gameObject.AddComponent<Steam2>();
            }
        }
        if (drivingWheel.transform.localRotation.eulerAngles.z > 315f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/0.png");
        else if (drivingWheel.transform.localRotation.eulerAngles.z > 270f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/45.png");
        else if (drivingWheel.transform.localRotation.eulerAngles.z > 225f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/90.png");
        else if (drivingWheel.transform.localRotation.eulerAngles.z > 180f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/135.png");
        else if (drivingWheel.transform.localRotation.eulerAngles.z > 135f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/180.png");
        else if (drivingWheel.transform.localRotation.eulerAngles.z > 90f)
        {
            readyForBrush = true;
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/225.png");
        }
        else if (drivingWheel.transform.localRotation.eulerAngles.z > 45f)
        {
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/270.png");
            brushSound();
        }
        else
        {
            readyForBrush = true;
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/315.png");
        }
    }
}


public class Steam : MonoBehaviour
{
    Rigidbody2D rb;

    private void Start()
    {
        rb = gameObject.AddComponent<Rigidbody2D>();
        rb.gravityScale = 0.01f;
        rb.velocity = transform.up * 1f;
        Destroy(gameObject, 8f);
    }

    private void Update()
    {
        transform.localScale += new Vector3(0.0005f * Time.deltaTime, 0.0005f * Time.deltaTime, 0.0005f * Time.deltaTime);
    }
}

public class Steam2 : MonoBehaviour
{
    Rigidbody2D rb;

    private void Start()
    {
        rb = gameObject.AddComponent<Rigidbody2D>();
        rb.gravityScale = 0.01f;
        rb.velocity = transform.right * -4f;
        Destroy(gameObject, 4f);
    }

    private void Update()
    {
        transform.localScale += new Vector3(0.0002f * Time.deltaTime, 0.0002f * Time.deltaTime, 0.0002f * Time.deltaTime);
    }
}

public class C62TenderSetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    JointSuspension2D jointMotor;
    WheelJoint2D[] wheels;
    BoxCollider2D[] cols;

    private void Awake()
    {
        cols = GetComponentsInChildren<BoxCollider2D>();
        foreach (BoxCollider2D col in cols)
        {
            Destroy(col);
        }
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCoupler.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));

        frontWheel_1.transform.position = transform.position + new Vector3(2.011f, -1.87f, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(-1.066f, -1.87f, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(-3.141f, -1.87f, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(4.094f, -1.87f, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();

        frontCoupler.transform.position = transform.position + new Vector3(4.34f, -0.644f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-4.777f, -1.4f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        jointMotor.frequency = 8f;
        jointMotor.dampingRatio = 0;

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
            wheel.suspension = jointMotor;
        }
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(1.9576f, -1.87f, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-1.043f, -1.87f, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-3.071f, -1.87f, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(3.9859f, -1.87f, 0);
    }
}

public class MarquetteTenderSetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    JointSuspension2D jointMotor;
    WheelJoint2D[] wheels;
    BoxCollider2D[] cols;

    private void Awake()
    {
        cols = GetComponentsInChildren<BoxCollider2D>();
        foreach (BoxCollider2D col in cols)
        {
            Destroy(col);
        }
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCoupler.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));

        frontWheel_1.transform.position = transform.position + new Vector3(3.234f, -2.439f, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(5.286f, -2.439f, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(-6.058f, -2.439f, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(-4f, -2.439f, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();

        frontCoupler.transform.position = transform.position + new Vector3(7.425f, -1.888f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-8.278f, -1.888f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        jointMotor.frequency = 8f;
        jointMotor.dampingRatio = 0;

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
            wheel.suspension = jointMotor;
        }
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(3.234f, -2.439f, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(5.286f, -2.439f, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-6.058f, -2.439f, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-4f, -2.439f, 0);
    }
}

public class BoxcarSetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    JointSuspension2D jointMotor;
    WheelJoint2D[] wheels;
    BoxCollider2D[] cols;

    private void Awake()
    {
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCoupler.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));

        frontWheel_1.transform.position = transform.position + new Vector3(5.69f, -2.37f, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(7.747f, -2.37f, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(-7.747f, -2.37f, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(-5.69f, -2.37f, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();

        frontCoupler.transform.position = transform.position + new Vector3(9.87f, -1.953f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-9.87f, -1.953f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();

        jointMotor.frequency = 8f;
        jointMotor.dampingRatio = 0;

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
            wheel.suspension = jointMotor;
        }
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(5.69f, -2.37f, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(7.747f, -2.37f, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-7.747f, -2.37f, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-5.69f, -2.37f, 0);
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }
    }
}


public class OrecarSetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    JointSuspension2D jointMotor;
    WheelJoint2D[] wheels;
    BoxCollider2D[] cols;

    private void Awake()
    {
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCoupler.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));

        frontWheel_1.transform.position = transform.position + new Vector3(4.32f, -2.37f, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(6.38f, -2.37f, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(-6.38f, -2.37f, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(-4.32f, -2.37f, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();

        frontCoupler.transform.position = transform.position + new Vector3(8.36f, -1.953f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-8.36f, -1.953f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        jointMotor.frequency = 8f;
        jointMotor.dampingRatio = 0;

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
            wheel.suspension = jointMotor;
        }
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(4.32f, -2.37f, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(6.38f, -2.37f, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-6.38f, -2.37f, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-4.32f, -2.37f, 0);
    }
}

public class BigBoySetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    public WheelJoint2D drivingFrontWheel, drivingBackWheel;
    private GameObject rod, rod2;
    float timer;
    private float mult = 1f;
    bool readyForBrushF = false, readyForBrushB = false;
    WheelJoint2D[] wheels;

    private void Awake()
    {
        /*ModAPI2.LoadSprite("rod/bigboy/0.png", mult);
        ModAPI2.LoadSprite("rod/bigboy/45.png", mult);
        ModAPI2.LoadSprite("rod/bigboy/90.png", mult);
        ModAPI2.LoadSprite("rod/bigboy/135.png", mult);
        ModAPI2.LoadSprite("rod/bigboy/180.png", mult);
        ModAPI2.LoadSprite("rod/bigboy/225.png", mult);
        ModAPI2.LoadSprite("rod/bigboy/270.png", mult);
        ModAPI2.LoadSprite("rod/bigboy/315.png", mult);*/
        GetComponent<SpriteRenderer>().sortingOrder = 3;
        var mainFrontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel/BigBoy.png"));
        var mainFrontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel/BigBoy.png"));
        var mainFrontWheel_3 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel/BigBoy.png"));
        var mainFrontWheel_4 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel/BigBoy.png"));
        var mainBackWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel/BigBoy.png"));
        var mainBackWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel/BigBoy.png"));
        var mainBackWheel_3 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel/BigBoy.png"));
        var mainBackWheel_4 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel/BigBoy.png"));
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCouplerFreight.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));
        rod = new GameObject();
        rod2 = new GameObject();

        mainFrontWheel_1.transform.position = transform.position + new Vector3(2.207f, -1.863f, 0);
        mainFrontWheel_1.transform.SetParent(transform);
        mainFrontWheel_1.AddComponent<WheelJoint2D>();
        mainFrontWheel_2.transform.position = transform.position + new Vector3(4.26f, -1.863f, 0);
        mainFrontWheel_2.transform.SetParent(transform);
        mainFrontWheel_2.AddComponent<WheelJoint2D>();
        mainFrontWheel_3.transform.position = transform.position + new Vector3(6.328f, -1.863f, 0);
        mainFrontWheel_3.transform.SetParent(transform);
        mainFrontWheel_3.AddComponent<WheelJoint2D>();
        mainFrontWheel_4.transform.position = transform.position + new Vector3(8.39f, -1.863f, 0);
        mainFrontWheel_4.transform.SetParent(transform);
        mainFrontWheel_4.AddComponent<WheelJoint2D>();

        mainBackWheel_1.transform.position = transform.position + new Vector3(-7.64f, -1.863f, 0);
        mainBackWheel_1.transform.SetParent(transform);
        mainBackWheel_1.AddComponent<WheelJoint2D>();
        mainBackWheel_2.transform.position = transform.position + new Vector3(-5.56f, -1.863f, 0);
        mainBackWheel_2.transform.SetParent(transform);
        mainBackWheel_2.AddComponent<WheelJoint2D>();
        mainBackWheel_3.transform.position = transform.position + new Vector3(-3.5f, -1.863f, 0);
        mainBackWheel_3.transform.SetParent(transform);
        mainBackWheel_3.AddComponent<WheelJoint2D>();
        mainBackWheel_4.transform.position = transform.position + new Vector3(-1.45f, -1.863f, 0);
        mainBackWheel_4.transform.SetParent(transform);
        mainBackWheel_4.AddComponent<WheelJoint2D>();

        frontWheel_1.transform.position = transform.position + new Vector3(9.92f, -2.207f, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(12.41f, -2.207f, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(-12.16f, -2.207f, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(-10.36f, -2.207f, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();

        rod.transform.position = transform.position + new Vector3(6.906f, -1.215f, 0);
        rod.transform.SetParent(transform);
        rod.gameObject.AddComponent<SpriteRenderer>();
        rod.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 2;
        if(transform.localScale.x < 0) //facing left
        {
            rod.transform.localScale = new Vector2(-rod.transform.localScale.x, rod.transform.localScale.y);
            rod.transform.localPosition = new Vector3(-rod.transform.localPosition.x, rod.transform.localPosition.y, rod.transform.localPosition.z);
        }

        rod2.transform.position = transform.position + new Vector3(-2.934f, -1.215f, 0);
        rod2.transform.SetParent(transform);
        rod2.gameObject.AddComponent<SpriteRenderer>();
        rod2.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 2;
        if (transform.localScale.x < 0) //facing left
        {
            rod2.transform.localScale = new Vector2(-rod2.transform.localScale.x, rod2.transform.localScale.y);
            rod2.transform.localPosition = new Vector3(-rod2.transform.localPosition.x, rod2.transform.localPosition.y, rod2.transform.localPosition.z);
        }

        frontCoupler.transform.position = transform.position + new Vector3(14.57f, -1.758f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-14.782f, -1.82f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
        }

        mainFrontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(2.207f, -1.863f, 0);
        mainFrontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(4.26f, -1.863f, 0);
        mainFrontWheel_3.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(6.328f, -1.863f, 0);
        mainFrontWheel_4.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(8.39f, -1.863f, 0);

        mainBackWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-7.64f, -1.863f, 0);
        mainBackWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-5.56f, -1.863f, 0);
        mainBackWheel_3.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-3.5f, -1.863f, 0);
        mainBackWheel_4.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-1.45f, -1.863f, 0);

        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(9.92f, -2.207f, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(12.41f, -2.207f, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-12.408f, -2.207f, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-10.582f, -2.207f, 0);
        drivingFrontWheel = mainFrontWheel_4.GetComponent<WheelJoint2D>();
        drivingBackWheel = mainBackWheel_4.GetComponent<WheelJoint2D>();
    }
    private void brushSound()
    {
        if (readyForBrushF)
        {
            readyForBrushF = false;
            GetComponent<Locomotive>().S_brush.Play();
        }
        if (readyForBrushB)
        {
            readyForBrushB = false;
            GetComponent<Locomotive>().S_brush.Play();
        }
    }
    private void Update()
    {
        if (drivingFrontWheel.transform.localRotation.eulerAngles.z > 315f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/0.png", mult);
        else if (drivingFrontWheel.transform.localRotation.eulerAngles.z > 270f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/45.png", mult);
        else if (drivingFrontWheel.transform.localRotation.eulerAngles.z > 225f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/90.png", mult);
        else if (drivingFrontWheel.transform.localRotation.eulerAngles.z > 180f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/135.png", mult);
        else if (drivingFrontWheel.transform.localRotation.eulerAngles.z > 135f)
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/180.png", mult);
        else if (drivingFrontWheel.transform.localRotation.eulerAngles.z > 90f)
        {
            readyForBrushF = true;
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/225.png", mult);
        }
        else if (drivingFrontWheel.transform.localRotation.eulerAngles.z > 45f)
        {
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/270.png", mult);
            brushSound();
        }
        else
        {
            readyForBrushF = true;
            rod.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/315.png", mult);
        }

        if (drivingBackWheel.transform.localRotation.eulerAngles.z > 315f)
            rod2.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/0.png", mult);
        else if (drivingBackWheel.transform.localRotation.eulerAngles.z > 270f)
            rod2.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/45.png", mult);
        else if (drivingBackWheel.transform.localRotation.eulerAngles.z > 225f)
            rod2.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/90.png", mult);
        else if (drivingBackWheel.transform.localRotation.eulerAngles.z > 180f)
            rod2.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/135.png", mult);
        else if (drivingBackWheel.transform.localRotation.eulerAngles.z > 135f)
            rod2.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/180.png", mult);
        else if (drivingBackWheel.transform.localRotation.eulerAngles.z > 90f)
        {
            readyForBrushB = true;
            rod2.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/225.png", mult);
        }
        else if (drivingBackWheel.transform.localRotation.eulerAngles.z > 45f)
        {
            rod2.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/270.png", mult);
            brushSound();
        }
        else
        {
            readyForBrushB = true;
            rod2.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("rod/bigboy/315.png", mult);
        }
        if (drivingFrontWheel.useMotor && drivingBackWheel.useMotor)
        {
            timer += Time.deltaTime;
            if (timer > 0.2f)
            {
                if (GetComponent<Locomotive>().coals == 0)
                    return;
                timer = 0;
                var steam = new GameObject();
                steam.transform.SetParent(transform);
                steam.transform.localPosition = new Vector3(9.02f, 2.78f, 0);
                steam.transform.SetParent(null);
                steam.gameObject.AddComponent<SpriteRenderer>();
                steam.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("steam/steam1.png");
                steam.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
                steam.gameObject.AddComponent<Steam>();
                var steam1 = new GameObject();
                steam1.transform.SetParent(transform);
                steam1.transform.localPosition = new Vector3(7.5f, 2.78f, 0);
                steam1.transform.SetParent(null);
                steam1.gameObject.AddComponent<SpriteRenderer>();
                steam1.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("steam/steam1.png");
                steam1.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
                steam1.gameObject.AddComponent<Steam>();
                var steam2 = new GameObject();
                steam2.transform.SetParent(transform);
                steam2.transform.localPosition = new Vector3(10.42f, -2.03f, 0);
                steam2.transform.SetParent(null);
                steam2.gameObject.AddComponent<SpriteRenderer>();
                steam2.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("steam/steam1.png");
                steam2.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 4;
                steam2.gameObject.AddComponent<Steam2>();
                var steam2_1 = new GameObject();
                steam2_1.transform.SetParent(transform);
                steam2_1.transform.localPosition = new Vector3(-0.3f, -2.03f, 0);
                steam2_1.transform.SetParent(null);
                steam2_1.gameObject.AddComponent<SpriteRenderer>();
                steam2_1.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("steam/steam1.png");
                steam2_1.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 4;
                steam2_1.gameObject.AddComponent<Steam2>();
            }
        }
    }
}
public class BigBoyTenderSetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    JointSuspension2D jointMotor;
    WheelJoint2D[] wheels;
    BoxCollider2D[] cols;

    private void Awake()
    {
        cols = GetComponentsInChildren<BoxCollider2D>();
        foreach (BoxCollider2D col in cols)
        {
            Destroy(col);
        }
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_3 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_4 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_5 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCoupler.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));

        frontWheel_1.transform.position = transform.position + new Vector3(2.917f, -2.237f, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(5.232f, -2.237f, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(-5.9f, -2.237f, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(-4.184f, -2.237f, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();
        backWheel_3.transform.position = transform.position + new Vector3(-2.414f, -2.237f, 0);
        backWheel_3.transform.SetParent(transform);
        backWheel_3.AddComponent<WheelJoint2D>();
        backWheel_4.transform.position = transform.position + new Vector3(-0.756f, -2.237f, 0);
        backWheel_4.transform.SetParent(transform);
        backWheel_4.AddComponent<WheelJoint2D>();
        backWheel_5.transform.position = transform.position + new Vector3(0.935f, -2.237f, 0);
        backWheel_5.transform.SetParent(transform);
        backWheel_5.AddComponent<WheelJoint2D>();

        frontCoupler.transform.position = transform.position + new Vector3(6.784f, -1.758f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-7.86f, -1.82f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        jointMotor.frequency = 8f;
        jointMotor.dampingRatio = 0;

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
            wheel.suspension = jointMotor;
        }
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(2.917f, -2.237f, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(5.232f, -2.237f, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-5.9f, -2.237f, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-4.184f, -2.237f, 0);
        backWheel_3.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-2.414f, -2.237f, 0);
        backWheel_4.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-0.756f, -2.237f, 0);
        backWheel_5.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(0.935f, -2.237f, 0);
    }
}

public class KtxSetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    JointSuspension2D jointMotor;
    WheelJoint2D[] wheels;
    BoxCollider2D[] cols;

    private void Awake()
    {
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCoupler.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));

        frontWheel_1.transform.position = transform.position + new Vector3(5.437f, -2.37f, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(8.977f, -2.37f, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(-11.06f, -2.37f, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(-7.514f, -2.37f, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();

        frontCoupler.transform.position = transform.position + new Vector3(13.082f, -1.82f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-13.246f, -1.82f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        jointMotor.frequency = 8f;
        jointMotor.dampingRatio = 0;

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
            wheel.suspension = jointMotor;
        }
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(5.437f, -2.37f, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(8.977f, -2.37f, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-11.06f, -2.37f, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-7.514f, -2.37f, 0);
    }
}

public class KtxWagonSetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    JointSuspension2D jointMotor;
    WheelJoint2D[] wheels;
    BoxCollider2D[] cols;

    private void Awake()
    {
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCoupler.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));

        frontWheel_1.transform.position = transform.position + new Vector3(6.456f, -2.37f, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(10f, -2.37f, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(-10f, -2.37f, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(-6.456f, -2.37f, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();

        frontCoupler.transform.position = transform.position + new Vector3(12.291f, -1.82f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-12.291f, -1.82f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        jointMotor.frequency = 8f;
        jointMotor.dampingRatio = 0;

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
            wheel.suspension = jointMotor;
        }
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(6.456f, -2.37f, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(10f, -2.37f, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-10f, -2.37f, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-6.456f, -2.37f, 0);
    }
}

public class WagonSetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    JointSuspension2D jointMotor;
    WheelJoint2D[] wheels;
    BoxCollider2D[] cols;

    private void Awake()
    {
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCoupler.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));

        frontWheel_1.transform.position = transform.position + new Vector3(7.312f, -2.371f, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(9.372f, -2.371f, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(-9.372f, -2.371f, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(-7.312f, -2.371f, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();

        frontCoupler.transform.position = transform.position + new Vector3(11.804f, -1.82f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-11.804f, -1.82f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        jointMotor.frequency = 8f;
        jointMotor.dampingRatio = 0;

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
            wheel.suspension = jointMotor;
        }
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(7.312f, -2.371f, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(9.372f, -2.371f, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-9.372f, -2.371f, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-7.312f, -2.371f, 0);
    }
}

public class CabooseSetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    JointSuspension2D jointMotor;
    WheelJoint2D[] wheels;
    BoxCollider2D[] cols;

    private void Awake()
    {
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCoupler.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));

        frontWheel_1.transform.position = transform.position + new Vector3(5.18f, -2.371f, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(7.236f, -2.371f, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(-7.236f, -2.371f, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(-5.18f, -2.371f, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();

        frontCoupler.transform.position = transform.position + new Vector3(9.3f, -1.94f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-9.3f, -1.94f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        jointMotor.frequency = 8f;
        jointMotor.dampingRatio = 0;

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
            wheel.suspension = jointMotor;
        }
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(5.18f, -2.371f, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(7.236f, -2.371f, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-7.236f, -2.371f, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-5.18f, -2.371f, 0);
    }
}

public class ArmSetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    JointSuspension2D jointMotor;
    WheelJoint2D[] wheels;
    BoxCollider2D[] cols;
    GameObject cannon;

    private void Awake()
    {
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCoupler.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));

        frontWheel_1.transform.position = transform.position + new Vector3(2.12f, -2.439f, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(3.81f, -2.439f, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(-4.06f, -2.439f, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(-2.35f, -2.439f, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();

        frontCoupler.transform.position = transform.position + new Vector3(5.93f, -1.888f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-5.93f, -1.888f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        jointMotor.frequency = 8f;
        jointMotor.dampingRatio = 0;

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
            wheel.suspension = jointMotor;
        }
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(2.12f, -2.439f, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(3.81f, -2.439f, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-4.06f, -2.439f, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-2.35f, -2.439f, 0);

        cannon = ModAPI.CreatePhysicalObject("cannon", ModAPI2.LoadSprite("armoredTrain_cannon.png"));
        //cannon.AddComponent<SpriteRenderer>();
        //cannon.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("gustav_cannon.png");
        cannon.transform.SetParent(transform);
        cannon.transform.localPosition = new Vector2(0, 0);
        //cannon.AddComponent<Rigidbody2D>();
        cannon.GetComponent<Rigidbody2D>().mass = 10;
        cannon.AddComponent<HingeJoint2D>();
        cannon.GetComponent<HingeJoint2D>().autoConfigureConnectedAnchor = false;
        cannon.GetComponent<HingeJoint2D>().connectedBody = GetComponent<Rigidbody2D>();
        cannon.GetComponent<HingeJoint2D>().anchor = new Vector2(3.76f, 2.26f);
        cannon.GetComponent<HingeJoint2D>().connectedAnchor = new Vector2(3.76f, 2.26f);
        Destroy(cannon.GetComponent<BoxCollider2D>());
        cannon.AddComponent<PolygonCollider2D>();
        cannon.GetComponent<HingeJoint2D>().useLimits = true;
        cannon.GetComponent<HingeJoint2D>().useMotor = true;
        JointAngleLimits2D limit = new JointAngleLimits2D();
        limit.max = 40;
        limit.min = -80;
        cannon.GetComponent<HingeJoint2D>().limits = limit;
        JointMotor2D motor = new JointMotor2D();
        motor.maxMotorTorque = 60000;
        cannon.GetComponent<HingeJoint2D>().motor = motor;
        cannon.AddComponent<ArmouredCannon>();
        cannon.transform.SetParent(null);
    }
}

public class ArmouredCannon : MonoBehaviour
{
    private void Awake()
    {
        gameObject.AddComponent<UseEventTrigger>().Action = () =>
        {
            shoot();
        };
    }

    public void shoot()
    {
        GameObject exp = ModAPI.CreateParticleEffect("Explosion", transform.position);
        exp.transform.SetParent(transform);
        exp.transform.localPosition = new Vector2(5.76f, 2.26f);
        exp.transform.SetParent(null);

        GameObject cannonBall = ModAPI.CreatePhysicalObject("ball", ModAPI2.LoadSprite("smallBullet.png"));
        cannonBall.transform.SetParent(transform);
        cannonBall.transform.localPosition = new Vector2(5.76f, 2.26f);
        cannonBall.transform.localRotation = transform.rotation;
        cannonBall.GetComponent<PhysicalBehaviour>().InitialMass = 8;
        cannonBall.GetComponent<PhysicalBehaviour>().TrueInitialMass = 8;
        cannonBall.GetComponent<PhysicalBehaviour>().rigidbody.mass = 8;
        cannonBall.GetComponent<Rigidbody2D>().velocity = transform.right * 200f;
        cannonBall.transform.SetParent(null);
        cannonBall.AddComponent<CannonBall>();
    }
}

public class CannonBall : MonoBehaviour
{
    private void OnCollisionEnter2D(Collision2D col)
    {
        explode();
    }

    private void explode()
    {
        ModAPI.CreateParticleEffect("BigZap", transform.position);
        Destroy(gameObject);
    }
}

public class GustavCannon : MonoBehaviour
{
    float timer = 5;
    private void Awake()
    {
        gameObject.AddComponent<UseEventTrigger>().Action = () =>
        {
            if(timer > 2)
                shoot();
        };
    }

    private void Update()
    {
        if (timer < 5)
            timer += Time.deltaTime;
    }

    public void shoot()
    {
        timer = 0;
        GameObject exp = ModAPI.CreateParticleEffect("EnormousExplosion", transform.position);
        exp.transform.SetParent(transform);
        exp.transform.localPosition = new Vector2(27.66f, 3.73f);
        exp.transform.SetParent(null);
        
        GameObject cannonBall = ModAPI.CreatePhysicalObject("ball", ModAPI2.LoadSprite("bullet.png"));
        cannonBall.transform.SetParent(transform);
        cannonBall.transform.localPosition = new Vector2(28.6f, 3.73f);
        cannonBall.transform.localRotation = transform.rotation;
        cannonBall.GetComponent<PhysicalBehaviour>().InitialMass = 1000;
        cannonBall.GetComponent<PhysicalBehaviour>().TrueInitialMass = 1000;
        cannonBall.GetComponent<PhysicalBehaviour>().rigidbody.mass = 1000;
        cannonBall.GetComponent<Rigidbody2D>().velocity = transform.right * 100f;
        cannonBall.transform.SetParent(null);
        cannonBall.AddComponent<GustavCannonBall>();
    }
}

public class GustavCannonBall : MonoBehaviour
{
    private void Update()
    {
        if (GetComponent<Rigidbody2D>().velocity.magnitude < 32)
            explode();
    }

    private void explode()
    {
        ModAPI.CreateParticleEffect("EnormousExplosion", transform.position);
        ExplosionCreator.CreatePulseExplosion(transform.position, 1000f, 80f, true, true);
        ExplosionCreator.CreateFragmentationExplosion(1, transform.position, 80f, 1000f, true, true, 1f);
        Destroy(gameObject);
    }
}

public class GustavSetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    JointSuspension2D jointMotor;
    WheelJoint2D[] wheels;
    GameObject cannon;

    private void Awake()
    {
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_3 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_4 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_5 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_6 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_7 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_8 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_9 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontWheel_10 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));

        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_3 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_4 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_5 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_6 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_7 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_8 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_9 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var backWheel_10 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("steamBackWheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCoupler.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));

        frontWheel_1.transform.position = transform.position + new Vector3(-21.871f, -5.319f, 0);
        frontWheel_2.transform.position = transform.position + new Vector3(-20.29f, -5.319f, 0);
        frontWheel_3.transform.position = transform.position + new Vector3(-18.72f, -5.319f, 0);
        frontWheel_4.transform.position = transform.position + new Vector3(-17.14f, -5.319f, 0);
        frontWheel_5.transform.position = transform.position + new Vector3(-15.55f, -5.319f, 0);
        frontWheel_6.transform.position = transform.position + new Vector3(-13.53f, -5.319f, 0);
        frontWheel_7.transform.position = transform.position + new Vector3(-11.95f, -5.319f, 0);
        frontWheel_8.transform.position = transform.position + new Vector3(-10.38f, -5.319f, 0);
        frontWheel_9.transform.position = transform.position + new Vector3(-8.79f, -5.319f, 0);
        frontWheel_10.transform.position = transform.position + new Vector3(-7.22f, -5.319f, 0);

        backWheel_1.transform.position = transform.position + new Vector3(8.54f, -5.319f, 0);
        backWheel_2.transform.position = transform.position + new Vector3(10.13f, -5.319f, 0);
        backWheel_3.transform.position = transform.position + new Vector3(11.7f, -5.319f, 0);
        backWheel_4.transform.position = transform.position + new Vector3(13.28f, -5.319f, 0);
        backWheel_5.transform.position = transform.position + new Vector3(14.85f, -5.319f, 0);
        backWheel_6.transform.position = transform.position + new Vector3(16.88f, -5.319f, 0);
        backWheel_7.transform.position = transform.position + new Vector3(18.46f, -5.319f, 0);
        backWheel_8.transform.position = transform.position + new Vector3(20.04f, -5.319f, 0);
        backWheel_9.transform.position = transform.position + new Vector3(21.62f, -5.319f, 0);
        backWheel_10.transform.position = transform.position + new Vector3(23.2f, -5.319f, 0);

        frontWheel_1.transform.SetParent(transform);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_3.transform.SetParent(transform);
        frontWheel_4.transform.SetParent(transform);
        frontWheel_5.transform.SetParent(transform);
        frontWheel_6.transform.SetParent(transform);
        frontWheel_7.transform.SetParent(transform);
        frontWheel_8.transform.SetParent(transform);
        frontWheel_9.transform.SetParent(transform);
        frontWheel_10.transform.SetParent(transform);

        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.AddComponent<WheelJoint2D>();
        frontWheel_3.AddComponent<WheelJoint2D>();
        frontWheel_4.AddComponent<WheelJoint2D>();
        frontWheel_5.AddComponent<WheelJoint2D>();
        frontWheel_6.AddComponent<WheelJoint2D>();
        frontWheel_7.AddComponent<WheelJoint2D>();
        frontWheel_8.AddComponent<WheelJoint2D>();
        frontWheel_9.AddComponent<WheelJoint2D>();
        frontWheel_10.AddComponent<WheelJoint2D>();

        backWheel_1.transform.SetParent(transform);
        backWheel_2.transform.SetParent(transform);
        backWheel_3.transform.SetParent(transform);
        backWheel_4.transform.SetParent(transform);
        backWheel_5.transform.SetParent(transform);
        backWheel_6.transform.SetParent(transform);
        backWheel_7.transform.SetParent(transform);
        backWheel_8.transform.SetParent(transform);
        backWheel_9.transform.SetParent(transform);
        backWheel_10.transform.SetParent(transform);

        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.AddComponent<WheelJoint2D>();
        backWheel_3.AddComponent<WheelJoint2D>();
        backWheel_4.AddComponent<WheelJoint2D>();
        backWheel_5.AddComponent<WheelJoint2D>();
        backWheel_6.AddComponent<WheelJoint2D>();
        backWheel_7.AddComponent<WheelJoint2D>();
        backWheel_8.AddComponent<WheelJoint2D>();
        backWheel_9.AddComponent<WheelJoint2D>();
        backWheel_10.AddComponent<WheelJoint2D>();

        frontCoupler.transform.position = transform.position + new Vector3(9.3f, -1.94f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-9.3f, -1.94f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        jointMotor.frequency = 8f;
        jointMotor.dampingRatio = 0;


        cannon = ModAPI.CreatePhysicalObject("cannon", ModAPI2.LoadSprite("gustav_cannon.png"));
        //cannon.AddComponent<SpriteRenderer>();
        //cannon.GetComponent<SpriteRenderer>().sprite = ModAPI2.LoadSprite("gustav_cannon.png");
        cannon.transform.SetParent(transform);
        cannon.transform.localPosition = new Vector2(0, 0);
        //cannon.AddComponent<Rigidbody2D>();
        cannon.GetComponent<Rigidbody2D>().mass = 60;
        cannon.AddComponent<HingeJoint2D>();
        cannon.GetComponent<HingeJoint2D>().autoConfigureConnectedAnchor = false;
        cannon.GetComponent<HingeJoint2D>().connectedBody = GetComponent<Rigidbody2D>();
        cannon.GetComponent<HingeJoint2D>().anchor = new Vector2(2.2f, 3.71f);
        cannon.GetComponent<HingeJoint2D>().connectedAnchor = new Vector2(2.2f, 3.71f);
        Destroy(cannon.GetComponent<BoxCollider2D>());
        cannon.AddComponent<PolygonCollider2D>();
        cannon.GetComponent<HingeJoint2D>().useLimits = true;
        cannon.GetComponent<HingeJoint2D>().useMotor = true;
        JointAngleLimits2D limit = new JointAngleLimits2D();
        limit.max = 10;
        limit.min = -90;
        cannon.GetComponent<HingeJoint2D>().limits = limit;
        JointMotor2D motor = new JointMotor2D();
        motor.maxMotorTorque = 400000;
        cannon.GetComponent<HingeJoint2D>().motor = motor;
        cannon.AddComponent<GustavCannon>();
        cannon.transform.SetParent(null);


        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 6;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 6;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 6;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
            wheel.suspension = jointMotor;
        }
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-21.871f, -5.319f, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-20.29f, -5.319f, 0);
        frontWheel_3.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-18.72f, -5.319f, 0);
        frontWheel_4.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-17.14f, -5.319f, 0);
        frontWheel_5.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-15.55f, -5.319f, 0);
        frontWheel_6.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-13.53f, -5.319f, 0);
        frontWheel_7.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-11.95f, -5.319f, 0);
        frontWheel_8.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-10.38f, -5.319f, 0);
        frontWheel_9.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-8.79f, -5.319f, 0);
        frontWheel_10.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(-7.22f, -5.319f, 0);

        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(8.54f, -5.319f, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(10.13f, -5.319f, 0);
        backWheel_3.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(11.7f, -5.319f, 0);
        backWheel_4.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(13.28f, -5.319f, 0);
        backWheel_5.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(14.85f, -5.319f, 0);
        backWheel_6.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(16.88f, -5.319f, 0);
        backWheel_7.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(18.46f, -5.319f, 0);
        backWheel_8.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(20.04f, -5.319f, 0);
        backWheel_9.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(21.62f, -5.319f, 0);
        backWheel_10.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(23.2f, -5.319f, 0);
    }
    void OnDestroy()
    {
        Destroy(cannon);
    }
}

public class BasiFlatbedSetUp : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    public float bogieDownY = -2f;
    public float frontBogieX = 8.615f;
    public float backBogieX = -8.615f;
    public float couplerInterval = 13.2f;
    public float interval = 2.873f;
    float suspensionValue = 24;
    JointSuspension2D jointMotor;
    WheelJoint2D[] wheels;
    BoxCollider2D[] cols;

    private void Awake()
    {
        cols = GetComponentsInChildren<BoxCollider2D>();
        foreach(BoxCollider2D col in cols)
        {
            Destroy(col);
        }
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCoupler.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));

        frontWheel_1.transform.position = transform.position + new Vector3(frontBogieX + interval, bogieDownY, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(frontBogieX, bogieDownY, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(backBogieX, bogieDownY, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(backBogieX - interval, bogieDownY, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();

        frontCoupler.transform.position = transform.position + new Vector3(couplerInterval, bogieDownY + 0.5f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-couplerInterval, bogieDownY + 0.5f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        jointMotor.frequency = suspensionValue;
        jointMotor.dampingRatio = 0;

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
            wheel.suspension = jointMotor;
        }
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(frontBogieX + interval, bogieDownY, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(frontBogieX, bogieDownY, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(backBogieX, bogieDownY, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(backBogieX - interval, bogieDownY, 0);
    }
}
public class BasiFlatbedSetUpDdamBbang : MonoBehaviour
{
    public WheelJoint2D wheelJoint;
    public float bogieDownY = -2f;
    public float frontBogieX = 3.442f;
    public float backBogieX = -3.442f;
    public float couplerInterval = 8f;
    public float interval = 2.873f;
    float suspensionValue = 24;
    JointSuspension2D jointMotor;
    WheelJoint2D[] wheels;
    BoxCollider2D[] cols;

    private void Awake()
    {
        cols = GetComponentsInChildren<BoxCollider2D>();
        foreach (BoxCollider2D col in cols)
        {
            Destroy(col);
        }
        var frontWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel.png"));
        var frontWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel.png"));
        var backWheel_1 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel.png"));
        var backWheel_2 = ModAPI.CreatePhysicalObject("Metal Wheel", ModAPI2.LoadSprite("wheel.png"));
        var frontCoupler = ModAPI.CreatePhysicalObject("frontCoupler", ModAPI2.LoadSprite("frontCoupler.png"));
        var backCoupler = ModAPI.CreatePhysicalObject("backCoupler", ModAPI2.LoadSprite("backCoupler.png"));

        frontWheel_1.transform.position = transform.position + new Vector3(frontBogieX + interval, bogieDownY, 0);
        frontWheel_1.transform.SetParent(transform);
        frontWheel_1.AddComponent<WheelJoint2D>();
        frontWheel_2.transform.position = transform.position + new Vector3(frontBogieX, bogieDownY, 0);
        frontWheel_2.transform.SetParent(transform);
        frontWheel_2.AddComponent<WheelJoint2D>();
        backWheel_1.transform.position = transform.position + new Vector3(backBogieX, bogieDownY, 0);
        backWheel_1.transform.SetParent(transform);
        backWheel_1.AddComponent<WheelJoint2D>();
        backWheel_2.transform.position = transform.position + new Vector3(backBogieX - interval, bogieDownY, 0);
        backWheel_2.transform.SetParent(transform);
        backWheel_2.AddComponent<WheelJoint2D>();

        frontCoupler.transform.position = transform.position + new Vector3(couplerInterval, bogieDownY + 0.5f, 0f);
        frontCoupler.transform.SetParent(transform);
        frontCoupler.AddComponent<CouplerBehaviour>();
        backCoupler.transform.position = transform.position + new Vector3(-couplerInterval, bogieDownY + 0.5f, 0f);
        backCoupler.transform.SetParent(transform);
        backCoupler.AddComponent<CouplerBehaviour>();
        if (transform.localScale.x < 0) //facing left
        {
            frontCoupler.transform.localPosition = new Vector3(-frontCoupler.transform.localPosition.x, frontCoupler.transform.localPosition.y, frontCoupler.transform.localPosition.z);
            frontCoupler.transform.localScale = new Vector2(-frontCoupler.transform.localScale.x, frontCoupler.transform.localScale.y);
            backCoupler.transform.localPosition = new Vector3(-backCoupler.transform.localPosition.x, backCoupler.transform.localPosition.y, backCoupler.transform.localPosition.z);
            backCoupler.transform.localScale = new Vector2(-backCoupler.transform.localScale.x, backCoupler.transform.localScale.y);
        }

        jointMotor.frequency = suspensionValue;
        jointMotor.dampingRatio = 0;

        wheels = GetComponentsInChildren<WheelJoint2D>();
        foreach (var wheel in wheels)
        {
            wheel.gameObject.GetComponent<PhysicalBehaviour>().InitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().TrueInitialMass = 4f;
            wheel.gameObject.GetComponent<PhysicalBehaviour>().rigidbody.mass = 4f;
            wheel.gameObject.GetComponent<BoxCollider2D>().enabled = false;
            wheel.gameObject.AddComponent<CircleCollider2D>();
            wheel.connectedBody = GetComponent<Rigidbody2D>();
            wheel.autoConfigureConnectedAnchor = false;
            wheel.gameObject.GetComponent<SpriteRenderer>().sortingOrder = 1;
            wheel.suspension = jointMotor;
        }
        frontWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(frontBogieX + interval, bogieDownY, 0);
        frontWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(frontBogieX, bogieDownY, 0);
        backWheel_1.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(backBogieX, bogieDownY, 0);
        backWheel_2.GetComponent<WheelJoint2D>().connectedAnchor = new Vector3(backBogieX - interval, bogieDownY, 0);
    }
}
public class reweighter : MonoBehaviour
{
    private void OnCollisionEnter2D(Collision2D col)
    {
        ModAPI.Notify("order:" + col.gameObject.GetComponent<SpriteRenderer>().sortingOrder);
        ModAPI.Notify("ID:" + col.gameObject.GetComponent<SpriteRenderer>().sortingLayerID);
        ModAPI.Notify("layerName:" + col.gameObject.GetComponent<SpriteRenderer>().sortingLayerName);
        ModAPI.Notify("sortingPoint:" + col.gameObject.GetComponent<SpriteRenderer>().spriteSortPoint);
    }
}